#include "DirectoryWatcher.h"

DirectoryWatcher::WatcherMap DirectoryWatcher::watchers;
HANDLE DirectoryWatcher::watchersMutex = CreateMutex(NULL, FALSE, "WatchersMutex");

RequestQueue RequestQueue::instance;


DirectoryWatcher::DirectoryWatcher(const string& dir, DirectoryWatcherHandler* hnd, bool watchSubDirs)
{
	directory = dir;
	handler = hnd;
	watchSubTree = watchSubDirs;
	status = ENABLED;
	directoryHandle = CreateFile(  directory.c_str(),
								   FILE_LIST_DIRECTORY,
								   FILE_SHARE_READ|FILE_SHARE_WRITE,
								   NULL,
								   OPEN_EXISTING,
								   FILE_FLAG_BACKUP_SEMANTICS|FILE_FLAG_OVERLAPPED,
								   NULL
								 );
}


DirectoryWatcher::~DirectoryWatcher()
{
	TRACE("Destroying watcher for: %s", directory.c_str());
	delete handler;
}


void DirectoryWatcher::dispatch(const string& fileName, DWORD action)
{
	switch(action)
	{
		case FILE_ACTION_ADDED:
			handler->fileAdded(fileName);
			break;
		case FILE_ACTION_REMOVED:
			handler->fileRemoved(fileName);
			break;
		case FILE_ACTION_MODIFIED:
			handler->fileModified(fileName);
			break;
		case FILE_ACTION_RENAMED_OLD_NAME:
			handler->fileRemoved(fileName);
			break;
		case FILE_ACTION_RENAMED_NEW_NAME:
			handler->fileAdded(fileName);
			break;
		default:
			break;
	}
}

void DirectoryWatcher::watchDirectory(const string& dirName, DirectoryWatcherHandler* handler, bool watchSubTree)
{
	TRACE("Adding watch for %s", dirName.c_str());

	DirectoryWatcher* dw = new DirectoryWatcher(dirName, handler, watchSubTree);
	
	unwatchDirectory(dirName);
	
	WaitForSingleObjectEx(watchersMutex, INFINITE, true);
	watchers.insert(WatcherMap::value_type(dirName, dw));
	ReleaseMutex(watchersMutex);

	RequestQueue::getInstance().add(dw);
}

void DirectoryWatcher::unwatchDirectory(const string& dirName)
{
	WaitForSingleObjectEx(watchersMutex, INFINITE, true);
	WatcherMap::const_iterator iter = watchers.find(dirName);
	if (iter != watchers.end())
	{
		TRACE("Removing watch for %s", dirName.c_str());
		DirectoryWatcher* dw = iter->second;
		watchers.erase(dirName);
		RequestQueue::getInstance().remove(dw);
	}
	ReleaseMutex(watchersMutex);
}

void DirectoryWatcher::unwatchAll()
{
	TRACE("Removing all watches");
	while (watchers.begin() != watchers.end())
	{
		unwatchDirectory(watchers.begin()->first);
	}
}

