#include "net_sourceforge_sunshade_filewatcher_NativeFileWatcher.h"
#include "DirectoryWatcher.h"

#include <iostream>
#include <sstream>
using namespace std;

static JavaVM* jvm;

class FileWatcherHandler : public DirectoryWatcherHandler
{
private:
	jobject javaHandler;

public:
	FileWatcherHandler(jobject hnd)
	{
		javaHandler = hnd;
	}
	
	~FileWatcherHandler()
	{
		JNIEnv* env;
		jvm->AttachCurrentThread((void**) &env, NULL);
		env->DeleteGlobalRef(javaHandler);		
		jvm->DetachCurrentThread();
		TRACE("Deleted FileWatcherHandler");
	}

	virtual void watchStarted(const string& dirName)
	{
		JNIEnv* env;
		jvm->AttachCurrentThread((void**) &env, NULL);

	    jclass cls = env->GetObjectClass(javaHandler);
		jmethodID callback =  env->GetMethodID(cls, "watchStarted", "(Ljava/lang/String;)V");

		if (callback)
		{
			jstring str = env->NewStringUTF(dirName.c_str());
			env->CallVoidMethod(javaHandler, callback, str);
		}

		jvm->DetachCurrentThread();
	}
		
	virtual void watchStopped(const string& dirName)
	{
		JNIEnv* env;
		jvm->AttachCurrentThread((void**) &env, NULL);

	    jclass cls = env->GetObjectClass(javaHandler);
		jmethodID callback =  env->GetMethodID(cls, "watchStopped", "(Ljava/lang/String;)V");

		if (callback)
		{
			jstring str = env->NewStringUTF(dirName.c_str());
			env->CallVoidMethod(javaHandler, callback, str);
		}
				
		jvm->DetachCurrentThread();
	}

	virtual void fileAdded(const string& fileName)
	{
		JNIEnv* env;
		jvm->AttachCurrentThread((void**) &env, NULL);

	    jclass cls = env->GetObjectClass(javaHandler);
		jmethodID callback =  env->GetMethodID(cls, "fileAdded", "(Ljava/lang/String;)V");

		if (callback)
		{
			jstring str = env->NewStringUTF(fileName.c_str());
			env->CallVoidMethod(javaHandler, callback, str);
		}
		
		jvm->DetachCurrentThread();
	}
	
	virtual void fileRemoved(const string& fileName)
	{
		JNIEnv* env;
		jvm->AttachCurrentThread((void**) &env, NULL);

	    jclass cls = env->GetObjectClass(javaHandler);
		jmethodID callback =  env->GetMethodID(cls, "fileRemoved", "(Ljava/lang/String;)V");

		if (callback)
		{
			jstring str = env->NewStringUTF(fileName.c_str());
			env->CallVoidMethod(javaHandler, callback, str);
		}
		
		jvm->DetachCurrentThread();
	}

	virtual void fileModified(const string& fileName)
	{
		JNIEnv* env;
		jvm->AttachCurrentThread((void**) &env, NULL);

	    jclass cls = env->GetObjectClass(javaHandler);
		jmethodID callback =  env->GetMethodID(cls, "fileModified", "(Ljava/lang/String;)V");

		if (callback)
		{
			jstring str = env->NewStringUTF(fileName.c_str());
			env->CallVoidMethod(javaHandler, callback, str);
		}
		
		jvm->DetachCurrentThread();
	}

};

/*
 * Class:     net_sourceforge_sunshade_filewatcher_NativeFileWatcher
 * Method:    jniinit
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_net_sourceforge_sunshade_filewatcher_NativeFileWatcher_jniinit
  (JNIEnv * env, jobject  instance)
{
	env->GetJavaVM(&jvm);
}

/*
 * Class:     net_sourceforge_sunshade_filewatcher_NativeFileWatcher
 * Method:    jnidestroy
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_net_sourceforge_sunshade_filewatcher_NativeFileWatcher_jnidestroy
  (JNIEnv * env, jobject instance)
{
	DirectoryWatcher::unwatchAll();
}

/*
 * Class:     net_sourceforge_sunshade_filewatcher_NativeFileWatcher
 * Method:    watchDirectory
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_net_sourceforge_sunshade_filewatcher_NativeFileWatcher_watchDirectory
  (JNIEnv * env, jobject instance, jstring path, jobject handler, jboolean watchSubDirs)
{
	const char *pathstr = env->GetStringUTFChars(path, 0);

	// memleak - need to clean up the handler
	FileWatcherHandler* cppHandler = new FileWatcherHandler(env->NewGlobalRef(handler));
	DirectoryWatcher::watchDirectory(string(pathstr),
									cppHandler,
									watchSubDirs);

	env->ReleaseStringUTFChars(path, pathstr);
}

/*
 * Class:     net_sourceforge_sunshade_filewatcher_NativeFileWatcher
 * Method:    unwatchDirectory
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_net_sourceforge_sunshade_filewatcher_NativeFileWatcher_unwatchDirectory
  (JNIEnv * env, jobject, jstring path)
{
	const char *pathstr = env->GetStringUTFChars(path, 0);
	DirectoryWatcher::unwatchDirectory(string(pathstr));
	env->ReleaseStringUTFChars(path, pathstr);
}

