package net.sourceforge.sunshade.filewatcher;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;


public class ChainedNativeChangeListener implements NativeChangeListener
{
	Collection delegates;

	public ChainedNativeChangeListener()
	{
		delegates = Collections.synchronizedCollection(new LinkedList());
	}
	
	public synchronized void addListener(NativeChangeListener listener)
	{
		delegates.add(listener);
	}
	
	public synchronized void removeListener(NativeChangeListener listener)
	{
		delegates.remove(listener);
	}
	
	public Collection getListeners()
	{
		return delegates;
	}
	
	public synchronized void watchStarted(String fileName)
	{
		if (FileWatcherPlugin.getDefault().isDebugging())
		{
			System.out.println("ChainedNativeChangeListener received a watchStarted notification for: " + fileName);
		}
		for (Iterator iter = delegates.iterator(); iter.hasNext();)
		{
			NativeChangeListener delegate = (NativeChangeListener) iter.next();
			delegate.watchStarted(fileName);
		}
	}

	public synchronized void watchStopped(String fileName)
	{
		if (FileWatcherPlugin.getDefault().isDebugging())
		{
			System.out.println("ChainedNativeChangeListener received a watchStopped notification for: " + fileName);
		}
		for (Iterator iter = delegates.iterator(); iter.hasNext();)
		{
			NativeChangeListener delegate = (NativeChangeListener) iter.next();
			delegate.watchStopped(fileName);
		}
	}

	public synchronized void fileAdded(String fileName)
	{
		for (Iterator iter = delegates.iterator(); iter.hasNext();)
		{
			NativeChangeListener delegate = (NativeChangeListener) iter.next();
			delegate.fileAdded(fileName);
		}
	}

	public synchronized void fileRemoved(String fileName)
	{
		for (Iterator iter = delegates.iterator(); iter.hasNext();)
		{
			NativeChangeListener delegate = (NativeChangeListener) iter.next();
			delegate.fileRemoved(fileName);
		}
	}

	public synchronized void fileModified(String fileName)
	{
		for (Iterator iter = delegates.iterator(); iter.hasNext();)
		{
			NativeChangeListener delegate = (NativeChangeListener) iter.next();
			delegate.fileModified(fileName);
		}
	}
	
}