package net.sourceforge.sunshade.filewatcher;

import java.util.Collection;
import java.util.LinkedList;

import net.sourceforge.sunshade.filewatcher.api.INativeChangeListener;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;

public class ExtensionManager
{
	private static ExtensionManager instance = new ExtensionManager();
	private Collection changeListeners;
	
	private ExtensionManager()
	{
		super();
		changeListeners = new LinkedList();
		readRegistry();
	}

	public static ExtensionManager getInstance()
	{
		return instance;
	}

	private void readRegistry()
	{
		IPluginRegistry registry = Platform.getPluginRegistry();
		IExtensionPoint point =
			registry.getExtensionPoint(
				"net.sourceforge.sunshade.filewatcher.changeListener");
		IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++)
		{
			IConfigurationElement[] elements =
				extensions[i].getConfigurationElements();
			for (int j = 0; j < elements.length; j++)
			{
				if ("changeListener".equals(elements[j].getName()))
				{
					try
					{
						INativeChangeListener changeListener =
							(INativeChangeListener) elements[j]
									.createExecutableExtension(
								"class");
						changeListeners.add(changeListener);
					}
					catch (CoreException e)
					{
						FileWatcherPlugin.log(IStatus.ERROR, "Could not create change listener", e);
					}
				}
			}
		}

	}
	
	public Collection getChangeListeners()
	{
		return changeListeners;
	}

}
