package net.sourceforge.sunshade.filewatcher;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * The main plugin class to be used in the desktop.
 */
public class FileWatcherPlugin extends AbstractUIPlugin implements IStartup
{
	//The shared instance.
	private static FileWatcherPlugin plugin;
	public static final String ID = "net.sourceforge.sunshade.filewatcher.FileWatcherPlugin";
	private ProjectDeltaListener projectListener;
	
	/**
	 * The constructor.
	 */
	public FileWatcherPlugin(IPluginDescriptor descriptor)
	{
		super(descriptor);
		plugin = this;
		projectListener = new ProjectDeltaListener();
	}

	/**
	 * Returns the shared instance.
	 */
	public static FileWatcherPlugin getDefault()
	{
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace()
	{
		return ResourcesPlugin.getWorkspace();
	}

	public static void log(int severity, String message, Throwable ex)
	{
		ILog log = FileWatcherPlugin.getDefault().getLog();
		log.log(new Status(severity, FileWatcherPlugin.ID, severity, message, ex));
	}
	
	public void shutdown() throws CoreException
	{
		super.shutdown();
		FileWatcherPlugin.getWorkspace().removeResourceChangeListener(projectListener);
		NativeFileWatcher.getInstance().destroy();
	}

	public void startup() throws CoreException
	{
		super.startup();
		
		// Add a watch for each project
		IProject[] projects = getWorkspace().getRoot().getProjects();
		for (int i = 0; i < projects.length; i++)
		{
			IProject project = projects[i];
			if (project.isOpen())
			{
				watchResource(project);
				LinkWatchManager.getInstance().watchLinks(project);
			}
		}
		
		// Watch for project additions/deletions
		FileWatcherPlugin.getWorkspace().addResourceChangeListener(
			projectListener,
			IResourceChangeEvent.PRE_DELETE | IResourceChangeEvent.PRE_CLOSE | IResourceChangeEvent.POST_CHANGE);
				
	}

	public void earlyStartup()
	{
	}
	
	private void watchResource(IResource resource)
	{
		if (FileWatcherPlugin.getDefault().isDebugging())
		{
			System.out.println("Adding a filesystem watch for resource: " + resource);
		}
		ModificationHandler modHandler = new ModificationHandler(resource);
		try
		{
			NativeFileWatcher.getInstance().addWatch(resource.getLocation().toOSString(), modHandler);
		}
		catch (FileWatcherException e)
		{
			FileWatcherPlugin.log(IStatus.ERROR, "Could not register a modification handler for project addition of: " + resource, e);
		}
	}

}
