package net.sourceforge.sunshade.filewatcher;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;

// Can't seem to get the OS path for a linked resource after that resource has
// been deleted (in the change event), so have to use this class to track all
// link creations and aid in the deletions
public class LinkWatchManager
{
	private static LinkWatchManager instance = new LinkWatchManager();
	private Map activeLinks;

	private LinkWatchManager()
	{
		activeLinks = new HashMap();
	}

	public static LinkWatchManager getInstance()
	{
		return instance;
	}
	
	public void watchLinks(IProject project)
	{
		try
		{
			IResource[] children = project.members();
			for (int i = 0; i < children.length; i++)
			{
				IResource resource = children[i];
				watchLink(resource);
			}
		}
		catch (CoreException e)
		{
			FileWatcherPlugin.log(IStatus.ERROR, "Could not add watches for links in project: " + project, e);
		}
	}
	
	public synchronized void watchLink(IResource resource)
	{
		if (resource.isLinked())
		{
			String osPath = resource.getLocation().toOSString();
			if (FileWatcherPlugin.getDefault().isDebugging())
			{
				System.out.println("Adding a filesystem watch for link: " + resource + ", " + osPath);
			}
			activeLinks.put(resource, osPath);
			ModificationHandler modHandler = new ModificationHandler(resource);
			try
			{
				NativeFileWatcher.getInstance().addWatch(osPath, modHandler);
			}
			catch (FileWatcherException e)
			{
				FileWatcherPlugin.log(IStatus.ERROR, "Could not register a modification handler for link: " + resource + ", " + osPath, e);
			}
		}
	}

	public synchronized void unwatchLink(IResource resource)
	{
		// have to save resource's os path for links as once the link is deleted we can't
		// get the os path from the resource
		String osPath = (String) activeLinks.remove(resource);
		if (osPath != null)
		{
			if (FileWatcherPlugin.getDefault().isDebugging())
			{
				System.out.println("Removing a filesystem watch for link: " + resource + ", " + osPath);
			}
			try
			{
				NativeFileWatcher.getInstance().removeWatch(osPath);
			}
			catch (FileWatcherException e)
			{
				FileWatcherPlugin.log(IStatus.ERROR, "Could not remove the modification handler for link: " + resource + ", " + osPath, e);
			}
		}
	}
}
