package net.sourceforge.sunshade.filewatcher;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class NativeFileWatcher
{

	private static NativeFileWatcher instance;
	private Map activeWatches;
	private Map activeNativeWatches;
	
	private NativeFileWatcher()
	{
		jniinit();
		activeWatches = new HashMap();
		activeNativeWatches = new HashMap();
	}

	public static synchronized NativeFileWatcher getInstance()
	{
		if (instance == null)
		{
			instance = new NativeFileWatcher();
		}
		return instance;
	}
	
	public synchronized void destroy()
	{
		jnidestroy();
		instance= null;
	}
	
	private native void jniinit();
	private native void jnidestroy();
	private native void watchDirectory(String dir, NativeChangeListener listener, boolean watchSubDirs);
	private native void unwatchDirectory(String dir);
	
	private ChainedNativeChangeListener getChainedListener(String dirname)
	{
		ChainedNativeChangeListener chain = (ChainedNativeChangeListener) activeNativeWatches.get(dirname);
		if (chain == null)
		{
			chain = new ChainedNativeChangeListener();
			chain.addListener(new NativeChangeAdapter()
			{
				public void watchStopped(String fileName)
				{
					activeNativeWatches.remove(fileName);
				}
			});
			activeNativeWatches.put(dirname, chain);
		}
		return chain;
	}

	public synchronized void addWatch(String dirOrFile, NativeChangeListener listener) throws FileWatcherException
	{
		try
		{

			String dirname = null;
			boolean recurse = false;
			NativeChangeListener handler = listener;
			
			File file = new File(dirOrFile).getCanonicalFile();
				
			if (file.isFile())
			{
				dirname = file.getParentFile().getPath();
				recurse = false;
				handler = new FileModifiedListener(file.getPath(), listener);
			}
			else
			{
				dirname = file.getPath();
				recurse = true;
			}

			if (activeWatches.containsKey(file.getPath()))
				return;
			activeWatches.put(file.getPath(), handler);

			ChainedNativeChangeListener chain = getChainedListener(dirname);

			chain.addListener(handler);
			if (chain.getListeners().size() == 2)
			{
				watchDirectory(dirname, chain, recurse);
			}
			else
			{
				handler.watchStarted(file.getPath());
			}
		}
		catch (IOException e)
		{
			throw new FileWatcherException("Unable to determine native directory name", e);
		}
	}

	public synchronized void removeWatch(String dirOrFile) throws FileWatcherException
	{
		try
		{
			String dirname = null;
			File file = new File(dirOrFile).getCanonicalFile();
			if (file.isFile())
			{
				File canonDir = file.getParentFile();
				String filename = canonDir.getName();
				dirname = canonDir.getPath();
			}
			else
			{
				dirname = file.getPath();
			}

			ChainedNativeChangeListener chain = getChainedListener(dirname);
			NativeChangeListener listener = (NativeChangeListener) activeWatches.remove(file.getPath());
			if (chain.getListeners().size() <= 2)
			{
				unwatchDirectory(dirname);
			}
			else
			{
				listener.watchStopped(file.getPath());
			}
			chain.removeListener(listener);
		}
		catch (IOException e)
		{
			throw new FileWatcherException("Unable to determine native directory name", e);
		}
	}


	static
	{
		System.loadLibrary("filewatcher");
	}

}
