package net.sourceforge.sunshade.filewatcher;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;

public class ProjectDeltaListener implements IResourceChangeListener
{
	private Map addedLinks = new HashMap();
	private IResourceDeltaVisitor deltaVisitor = new IResourceDeltaVisitor()
	{
		public boolean visit(IResourceDelta delta) throws CoreException
		{
			IResource resource = delta.getResource();
			switch(resource.getType())
			{
				case IResource.ROOT :
					return true;
				case IResource.PROJECT :
					IProject project = (IProject) resource;
					switch(delta.getKind())
					{
						case IResourceDelta.ADDED:
							watchResource(project);
							break;
						case IResourceDelta.CHANGED:
							if ((delta.getFlags() & IResourceDelta.OPEN) != 0 && project.isOpen())
							{
								watchResource(project);
							}
					}
					return true;
				default:
					switch(delta.getKind())
					{
						case IResourceDelta.ADDED:
							LinkWatchManager.getInstance().watchLink(resource);
							break;
						case IResourceDelta.REMOVED:
							LinkWatchManager.getInstance().unwatchLink(resource);
							break;
					}
			}
			return false;
		}
	};

	public ProjectDeltaListener()
	{
		super();
	}

	public void resourceChanged(IResourceChangeEvent event)
	{
		try
		{
			switch (event.getType())
			{
				case IResourceChangeEvent.PRE_CLOSE :
				case IResourceChangeEvent.PRE_DELETE :
					IResource resource = event.getResource();
					if (resource.getType() == IResource.PROJECT)
					{
						unwatchResource(resource);
					}
					break;
				case IResourceChangeEvent.POST_CHANGE :
					event.getDelta().accept(deltaVisitor);
					break;
				default :
					break;
			}
		}
		catch (CoreException e)
		{
			FileWatcherPlugin.log(
				IStatus.ERROR,
				"Unable to process resource changed events",
				e);
		}
	}
	
	private void watchResource(IResource resource)
	{
		if (FileWatcherPlugin.getDefault().isDebugging())
		{
			System.out.println("Adding a filesystem watch for resource: " + resource);
		}
		ModificationHandler modHandler = new ModificationHandler(resource);
		try
		{
			NativeFileWatcher.getInstance().addWatch(resource.getLocation().toOSString(), modHandler);
		}
		catch (FileWatcherException e)
		{
			FileWatcherPlugin.log(IStatus.ERROR, "Could not register a modification handler for project addition of: " + resource, e);
		}
	}

	private void unwatchResource(IResource resource)
	{
		if (FileWatcherPlugin.getDefault().isDebugging())
		{
			System.out.println("Removing the filesystem watch for resource: " + resource);
		}
		try
		{
			NativeFileWatcher.getInstance().removeWatch(resource.getLocation().toOSString());
		}
		catch (FileWatcherException e)
		{
			FileWatcherPlugin.log(IStatus.ERROR, "Could not remove the modification handler for project deletion of: " + resource, e);
		}
	}
	
}
